; StringInput
; Inputs: hl -> Location to store the string to
;         de = penrow/pencol to start display at
;         b = maximum number of characters
;         c = inputflags
;
; StringInputST
; Runs stringinput, but uses a structure instead of register inputs.
; Inputs: hl -> StringInput structure
;
; StringInput Structure:
; .db inputflags
; .db maxchars
; .db col,row
; .dw wheretostore
;
; inputflags:                       SET (nz)                            RESET (z)
;  Bit 0: SI_BigFont                Use the large font                  Use the small font
;  Bit 1: SI_NoCursor               Do not display cursor               Display cursor
;  Bit 2: SI_NoDisp                 Do not display typed in text        Display text
;  Bit 3: SI_NumbersDef             Numbers are input by default        Letters are input by default
;  Bit 4: SI_NoAlpha                Alpha does nothing                  Alpha switches between letters/numbers
;  Bit 5: SI_NoClearDel             Clear/delete do nothing             Clear/Delete work normally
;  Bit 6: SI_NullAllowed            Allow zero-length strings           Do not allow zero-length strings
;  Bit 7: SI_UseDefault             Default string is at (hl)           No default string
;
; Comments:
;  You may set textinverse or texterasebelow before running this routine if you wish.
;

SI_BigFont        equ   0
SI_NoCursor       equ   1
SI_NoDisp         equ   2
SI_NumbersDef     equ   3
SI_NoAlpha        equ   4
SI_NoClearDel     equ   5
SI_NullAllowed    equ   6
SI_UseDefault     equ   7

SI_CursorShowing  equ   0     ; overwrites BigFont flag

guiStringInputST:
 ld c,(hl)
 inc hl
 ld b,(hl)
 inc hl
 ld e,(hl)
 inc hl
 ld d,(hl)
 inc hl
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
guiStringInput:
 set plotLoc,(iy+plotflags)
 res textwrite,(iy+sgrflags)
 res fracDrawLFont,(iy+fontFlags)
 bit SI_BigFont,c
 jr z,guiStringInputSmall
 set fracDrawLFont,(iy+fontFlags)
guiStringInputSmall:
 ld (pencol),de
 res SI_CursorShowing,c
 ld d,b
 bit SI_UseDefault,c
 jr nz,guiStringInputIsDefault
 ld (hl),0
guiStringInputIsDefault:
 ld a,(hl)
 or a
 jr z,guiStringInputLoop
 inc hl
 dec b
 push hl
 push de
 push bc
 cp 20h
 jr nz,guiStringInputDefNotSpace
 bit fracDrawLFont,(iy+fontflags)
 jr nz,guiStringInputDefNotSpace
 ld a,6
guiStringInputDefNotSpace:
 B_CALL _vputmap
 pop bc
 pop de
 pop hl
 jr guiStringInputIsDefault
guiStringInputLoop:
 bit SI_NoCursor,c
 call z,guiStringInputCursor
 ei
 halt
 push hl
 B_CALL _getcsc
 pop hl
 or a
 jr z,guiStringInputLoop
 push af
 bit SI_CursorShowing,c
 call nz,guiStringInputCursor
 pop af
 cp skDel
 jr z,guiStringInputDelete
 cp skEnter
 jr z,guiStringInputEnter
 cp skClear
 jr z,guiStringInputClear
 cp skAlpha
 jr z,guiStringInputAlpha
 push de
 ld e,a
 ld d,0
 push hl
 ld hl,guiStringInputNumTable-1
 bit SI_NumbersDef,c
 jr nz,guiStringInputNumber
 ld hl,guiStringInputLetTable-1
guiStringInputNumber:
 add hl,de
 ld a,(hl)
 pop hl
 pop de
 or a
 jr z,guiStringInputLoop
 inc b
 dec b
 jr z,guiStringInputLoop
 ld (hl),a
 inc hl
 dec b
 bit SI_NoDisp,c
 jr nz,guiStringInputLoop
 cp 20h
 jr nz,guiStringInputNotSpace
 bit fracDrawLFont,(iy+fontflags)
 jr nz,guiStringInputNotSpace
 ld a,6
guiStringInputNotSpace:
 push hl
 push de
 B_CALL _vputmap
 pop de
 pop hl
 jr guiStringInputLoop
guiStringInputAlpha:
 bit SI_NoAlpha,c
 jr nz,guiStringInputLoop
 ld a,c
 xor 1<<SI_NumbersDef
 ld c,a
 jr guiStringInputLoop
guiStringInputEnter:
 bit 1,c
 call nz,guiStringInputCursor
 bit SI_NullAllowed,c
 jr nz,guiStringInputSkipNullCheck
 ld a,b
 cp d
 jr z,guiStringInputLoop
guiStringInputSkipNullCheck:
 ld (hl),0
 res fracdrawlfont,(iy+fontflags)
 ret
guiStringInputDelete:
 bit SI_NoClearDel,c
 jr nz,guiStringInputLoop
 ld a,b
 cp d
 jp z,guiStringInputLoop
 call guiStringInputBackspace
 jp guiStringInputLoop
guiStringInputClear:
 bit SI_NoClearDel,c
 jp nz,guiStringInputLoop
guiStringInputClearLoop:
 ld a,b
 cp d
 jp z,guiStringInputLoop
 call guiStringInputBackspace
 jr guiStringInputClearLoop
guiStringInputBackspace:
 dec hl
 inc b
 bit SI_NoDisp,c
 jp nz,guiStringInputLoop
 push hl
 ld hl,(pencol)
 ld a,l
 sub 4
 bit fracDrawLFont,(iy+fontflags)
 jr nz,guiStringInputBigBack
 ld l,a
 ld (pencol),hl
 ld a,6        ;four spaces
guiStringInputBigBack2:
 push hl
 push de
 B_CALL _vputmap
 pop de
 pop hl
 ld (pencol),hl
 pop hl
 ret
guiStringInputBigBack:
 sub 2
 ld l,a
 ld (pencol),hl
 ld a,20h
 jr guiStringInputBigBack2
guiStringInputCursor:
 ld a,c
 xor 1<<SI_CursorShowing
 ld c,a
 push hl
 push bc
 push de
 ld a,(pencol)
 ld d,a
 ld b,a
 ld a,(penrow)
 xor 3fh
 ld c,a
 sub 6
 bit fracDrawLFont,(iy+fontflags)
 jr z,guiStringInputCursorSmall
 dec a
 inc c
guiStringInputCursorSmall:
 ld e,a
 ld h,2
 B_CALL _iline
 pop de
 pop bc
 pop hl
 ret

guiStringInputNumTable:
 db 00h,00h,00h,00h,00h,00h,00h,00h
 db 00h,2Bh,2Dh,00h,2Fh,5Eh,00h,00h
 db 1Ah,33h,36h,39h,00h,00h,00h,00h
 db 0Ch,32h,35h,38h,00h,00h,00h,00h
 db 30h,31h,34h,37h,00h,00h,00h,00h
 db 00h,00h,00h,00h,00h,00h,00h,00h
 db 00h,00h,00h,00h,00h,00h,00h,00h

guiStringInputLetTable:
 db 00h,00h,00h,00h,00h,00h,00h,00h
 db 00h,22h,57h,52h,4Dh,48h,00h,00h
 db 3Fh,5Bh,56h,51h,4Ch,47h,00h,00h
 db 00h,5Ah,55h,50h,4Bh,46h,43h,00h
 db 20h,59h,54h,4Fh,4Ah,45h,42h,00h
 db 00h,58h,53h,4Eh,49h,44h,41h,00h
 db 00h,00h,00h,00h,00h,00h,00h,00h